#define WIN32_LEAN_AND_MEAN
#include <Windows.h>
#include <cassert>
#include <sstream>
#include <string>

#include <detours.h>

using namespace std;

#define APP_NAME	"Unity Trial Reset"
#define APP_EXE		"UnityTrialReset.exe"
#define HELPER_DLL	"UnityTrialResetHelper.dll"
#define UNITY_EXE	"Unity.exe"


string GetExeDirectory()
{
	static string folder;
	if (folder.empty()) {
		char buf[260];
		GetModuleFileName(GetModuleHandle(NULL), buf, sizeof(buf));
		folder = buf;
		int idx = folder.rfind('\\');
		assert(idx != -1);
		folder = folder.substr(0, idx + 1);
	}
	return folder;
}

string GetHelperDll()
{
	return GetExeDirectory() + HELPER_DLL;
}

string GetUnityExe()
{
	return GetExeDirectory() + UNITY_EXE;
}


int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, PSTR pCmdLine, int nCmdShow)
{
	UNREFERENCED_PARAMETER(hInstance);
	UNREFERENCED_PARAMETER(hPrevInstance);
	UNREFERENCED_PARAMETER(pCmdLine);
	UNREFERENCED_PARAMETER(nCmdShow);

	if (GetFileAttributes(GetHelperDll().c_str()) == INVALID_FILE_ATTRIBUTES) {
		MessageBox(NULL, HELPER_DLL " must be placed in the same folder as " APP_EXE, APP_NAME, MB_OK | MB_ICONERROR);
		return 1;
	}

	if (GetFileAttributes(GetUnityExe().c_str()) == INVALID_FILE_ATTRIBUTES) {
		MessageBox(NULL, APP_EXE " must be placed in the same folder as " UNITY_EXE, APP_NAME, MB_OK | MB_ICONERROR);
		return 1;
	}

	HMODULE hDll = LoadLibraryEx(GetHelperDll().c_str(), NULL, DONT_RESOLVE_DLL_REFERENCES);
	if (hDll == NULL) {
		MessageBox(NULL, "Failed to load " HELPER_DLL, APP_NAME, MB_OK | MB_ICONERROR);
		return 1;
	}
	FreeLibrary(hDll);


	STARTUPINFO si = { 0 };
	si.cb = sizeof(si);
	PROCESS_INFORMATION pi = { 0 };
	DWORD dwFlags = CREATE_DEFAULT_ERROR_MODE | CREATE_SUSPENDED;

	if (!DetourCreateProcessWithDllEx(GetUnityExe().c_str(), NULL, NULL, NULL, TRUE,
										dwFlags, NULL, GetExeDirectory().c_str(),
										&si, &pi, GetHelperDll().c_str(), NULL)) {
		MessageBox(NULL, "DLL hooking failed.", APP_NAME, MB_OK | MB_ICONERROR);
		ExitProcess(1);
	}

	ResumeThread(pi.hThread);

	WaitForSingleObject(pi.hProcess, INFINITE);

	DWORD dwResult = 0;
	if (!GetExitCodeProcess(pi.hProcess, &dwResult)) {
		MessageBox(NULL, "Failed to obtain return code from " UNITY_EXE, APP_NAME, MB_OK | MB_ICONERROR);
		return 1;
	}

	ostringstream oss;
	oss << "Return code from " UNITY_EXE ": " << dwResult;
	MessageBox(NULL, oss.str().c_str(), APP_NAME, MB_OK | MB_ICONINFORMATION);
	return dwResult;
}
