#define WIN_LEAN_AND_MEAN
#include <Windows.h>
#include <stdio.h>
#include <detours.h>


#define LOG_FILE		"C:\\ProgramData\\Unity\\UnityTrialReset.log"
#define PRODUCT_ID_FILE	"C:\\ProgramData\\Unity\\WinProductId.txt"

static LONG (WINAPI * ORI_RegGetValueA)(HKEY hkey, LPCSTR lpSubKey, LPCSTR lpValue, DWORD dwFlags, LPDWORD pdwType, PVOID pvData, LPDWORD pcbData) = RegGetValueA;
static LONG (WINAPI * ORI_RegQueryValueA)(HKEY hKey, LPCSTR lpSubKey, LPSTR lpValue, PLONG lpcbValue) = RegQueryValueA;
static LONG (WINAPI * ORI_RegQueryValueExA)(HKEY hKey, LPCSTR lpValueName, LPDWORD lpReserved, LPDWORD lpType, LPBYTE lpData, LPDWORD lpcbData) = RegQueryValueExA;
static LONG (WINAPI * ORI_RegEnumValueA)(HKEY hKey, DWORD dwIndex, LPSTR lpValueName, LPDWORD lpcchValueName, LPDWORD lpReserved, LPDWORD lpType, LPBYTE lpData, LPDWORD lpcbData) = RegEnumValueA;
static LONG (WINAPI * ORI_RegGetValueW)(HKEY hkey, LPCWSTR lpSubKey, LPCWSTR lpValue, DWORD dwFlags, LPDWORD pdwType, PVOID pvData, LPDWORD pcbData) = RegGetValueW;
static LONG (WINAPI * ORI_RegQueryValueW)(HKEY hKey, LPCWSTR lpSubKey, LPWSTR lpValue, PLONG lpcbValue) = RegQueryValueW;
static LONG (WINAPI * ORI_RegQueryValueExW)(HKEY hKey, LPCWSTR lpValueName, LPDWORD lpReserved, LPDWORD lpType, LPBYTE lpData, LPDWORD lpcbData) = RegQueryValueExW;
static LONG (WINAPI * ORI_RegEnumValueW)(HKEY hKey, DWORD dwIndex, LPWSTR lpValueName, LPDWORD lpcchValueName, LPDWORD lpReserved, LPDWORD lpType, LPBYTE lpData, LPDWORD lpcbData) = RegEnumValueW;

static FILE* log = NULL;


LONG WINAPI MY_RegGetValueA(HKEY hKey, LPCSTR lpSubKey, LPCSTR lpValue, DWORD dwFlags, LPDWORD pdwType, PVOID pvData, LPDWORD pcbData)
{
	LONG result = ORI_RegGetValueA(hKey, lpSubKey, lpValue, dwFlags, pdwType, pvData, pcbData);
	fwprintf(log, L"MY_RegGetValueA\tSubkey: %S,\tValue name: %S,\tData: %S\n", lpSubKey, lpValue, pvData);
	fflush(log); 
	return result;
}

LONG WINAPI MY_RegQueryValueA(HKEY hKey, LPCSTR lpSubKey, LPSTR lpValue, PLONG lpcbValue)
{
	LONG result = ORI_RegQueryValueA(hKey, lpSubKey, lpValue, lpcbValue);
	fwprintf(log, L"MY_RegQueryValueA\tSubkey: %S,\tDefault value: %S\n", lpSubKey, lpValue);
	fflush(log); 
	return result;
}

LONG WINAPI MY_RegQueryValueExA(HKEY hKey, LPCSTR lpValueName, LPDWORD lpReserved, LPDWORD lpType, LPBYTE lpData, LPDWORD lpcbData)
{
	LONG result = ORI_RegQueryValueExA(hKey, lpValueName, lpReserved, lpType, lpData, lpcbData);
	fwprintf(log, L"MY_RegQueryValueExA\tValue name: %S,\tData: %S\n", lpValueName, lpData);
	fflush(log);
	return result;
}

LONG WINAPI MY_RegEnumValueA(HKEY hKey, DWORD dwIndex, LPSTR lpValueName, LPDWORD lpcchValueName, LPDWORD lpReserved, LPDWORD lpType, LPBYTE lpData, LPDWORD lpcbData)
{
	LONG result = ORI_RegEnumValueA(hKey, dwIndex, lpValueName, lpcchValueName, lpReserved, lpType, lpData, lpcbData);
	fwprintf(log, L"MY_RegEnumValueA\tValue name: %S,\tData: %S\n", lpValueName, lpData);
	fflush(log);
	return result;
}

LONG WINAPI MY_RegGetValueW(HKEY hKey, LPCWSTR lpSubKey, LPCWSTR lpValue, DWORD dwFlags, LPDWORD pdwType, PVOID pvData, LPDWORD pcbData)
{
	LONG result = ORI_RegGetValueW(hKey, lpSubKey, lpValue, dwFlags, pdwType, pvData, pcbData);
	fwprintf(log, L"MY_RegGetValueW\tSubkey: %s,\tValue name: %s,\tData: %s\n", lpSubKey, lpValue, pvData);
	fflush(log);
	return result;
}

LONG WINAPI MY_RegQueryValueW(HKEY hKey, LPCWSTR lpSubKey, LPWSTR lpValue, PLONG lpcbValue)
{
	LONG result = ORI_RegQueryValueW(hKey, lpSubKey, lpValue, lpcbValue);
	fwprintf(log, L"MY_RegQueryValueW\tSubkey: %s,\tDefault value: %s\n", lpSubKey, lpValue);
	fflush(log);
	return result;
}

LONG WINAPI MY_RegQueryValueExW(HKEY hKey, LPCWSTR lpValueName, LPDWORD lpReserved, LPDWORD lpType, LPBYTE lpData, LPDWORD lpcbData)
{
	LONG result = ORI_RegQueryValueExW(hKey, lpValueName, lpReserved, lpType, lpData, lpcbData);
	if (wcscmp(lpValueName, L"ProductId") == 0) {
		FILE* fpid = fopen(PRODUCT_ID_FILE, "r");
		if (fpid) {
			char buf[256] = { 0 };
			fgets(buf, 256, fpid);
			swprintf((wchar_t*)lpData, *lpcbData / 2, L"%S", buf);
			fclose(fpid);
		}
	}
	fwprintf(log, L"MY_RegQueryValueExW\tValue name: %s,\tData: %s\n", lpValueName, lpData);
	fflush(log);
	return result;
}

LONG WINAPI MY_RegEnumValueW(HKEY hKey, DWORD dwIndex, LPWSTR lpValueName, LPDWORD lpcchValueName, LPDWORD lpReserved, LPDWORD lpType, LPBYTE lpData, LPDWORD lpcbData)
{
	LONG result = ORI_RegEnumValueW(hKey, dwIndex, lpValueName, lpcchValueName, lpReserved, lpType, lpData, lpcbData);
	fwprintf(log, L"MY_RegEnumValueW\tValue name: %s,\tData: %s\n", lpValueName, lpData);
	fflush(log);
	return result;
}


BOOL WINAPI DllMain(HINSTANCE hinstDLL, DWORD dwReason, LPVOID lpReserved)
{
	UNREFERENCED_PARAMETER(hinstDLL);
	UNREFERENCED_PARAMETER(lpReserved);

	if (DetourIsHelperProcess()) return TRUE;

	if (dwReason == DLL_PROCESS_ATTACH) {
		log = fopen(LOG_FILE, "w");
		if (!log) {
			MessageBox(NULL, "Failed to open log file.", "Error - Unity Trial Reset", MB_OK | MB_ICONERROR);
		}
		
		DetourTransactionBegin();
		DetourUpdateThread(GetCurrentThread());
		DetourAttach(&(PVOID&)ORI_RegGetValueA, MY_RegGetValueA);
		DetourAttach(&(PVOID&)ORI_RegQueryValueA, MY_RegQueryValueA);
		DetourAttach(&(PVOID&)ORI_RegQueryValueExA, MY_RegQueryValueExA);
		DetourAttach(&(PVOID&)ORI_RegEnumValueA, MY_RegEnumValueA);
		DetourAttach(&(PVOID&)ORI_RegGetValueW, MY_RegGetValueW);
		DetourAttach(&(PVOID&)ORI_RegQueryValueW, MY_RegQueryValueW);
		DetourAttach(&(PVOID&)ORI_RegQueryValueExW, MY_RegQueryValueExW);
		DetourAttach(&(PVOID&)ORI_RegEnumValueW, MY_RegEnumValueW);
		
		LONG error = DetourTransactionCommit();
		if (error != NO_ERROR) {
			char buf[256];
			sprintf_s(buf, "Error detouring EntryPoint() : %d\n", error);
			MessageBox(NULL, buf, "Error - Unity Trial Reset", MB_OK | MB_ICONERROR);
		}

	} else if (dwReason == DLL_PROCESS_DETACH) {
		DetourTransactionBegin();
		DetourUpdateThread(GetCurrentThread());
		DetourDetach(&(PVOID&)ORI_RegGetValueA, MY_RegGetValueA);
		DetourDetach(&(PVOID&)ORI_RegQueryValueA, MY_RegQueryValueA);
		DetourDetach(&(PVOID&)ORI_RegQueryValueExA, MY_RegQueryValueExA);
		DetourDetach(&(PVOID&)ORI_RegEnumValueA, MY_RegEnumValueA);
		DetourDetach(&(PVOID&)ORI_RegGetValueW, MY_RegGetValueW);
		DetourDetach(&(PVOID&)ORI_RegQueryValueW, MY_RegQueryValueW);
		DetourDetach(&(PVOID&)ORI_RegQueryValueExW, MY_RegQueryValueExW);
		DetourDetach(&(PVOID&)ORI_RegEnumValueW, MY_RegEnumValueW);
		DetourTransactionCommit();

		fclose(log);
	}
	return TRUE;
}
